/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import org.zaval.awt.event.*;

/**
 * <font face="Arial" size="2">
 * This cursor controller class can be used to control cursor type for the specified
 * target component. Use the controller according to the following:
 * <ul>
 *   <li>
 *      Set a cursor info for the controller using <code>setCursorInfo</code> method. The
 *      cursor info defines cursor type for the target component depending on the mouse
 *      pointer location.
 *   </li>
 *   <li>
 *     Use <code>addActionListener</code> and <code>removeActionListener</code> methods
 *     to listen when the cursor has been changed for the target component.
 *   </li>
 *   <li>
 *     Use <code>getCursorType</code> method to get current cursor type.
 *   </li>
 * </ul>
 * <p>
 * The controller registers itself as MouseMotionListener and MouseListener listeners of
 * the target component to control cursor type.
 * </font>
 */
public class CursorController
extends Controller
implements MouseMotionListener, MouseListener
{
   private CursorInfo      cursorInfo;
   private Cursor          defaultCursor;
   private ActionSupport   actionSupport;
   private int        cursorType;
   private boolean    isDragged, isExited, isPressed;

  /**
   * <font face="Arial" size="2">
   * Constructs a new cursor controller with the specified target component.
   * @param <code>target</code> the specified target component.
   * </font>
   */
   public CursorController(Component target) {
     this (target, new InsetsCursorInfo());
   }

  /**
   * <font face="Arial" size="2">
   * Constructs a new cursor controller with the specified target component and
   * the given cursor info.
   * @param <code>target</code> the specified target component.
   * @param <code>i</code> the specified cursor info.
   * </font>
   */
   public CursorController(Component target, CursorInfo i) {
     super(target);
     setCursorInfo(i);
   }

  /**
   * <font face="Arial" size="2">
   * Sets the specified cursor info. The cursor info defines mouse cursor type depending
   * on the the mouse pointer location relatively the target component.
   * @param <code>i</code> the specified cursor info.
   * </font>
   */
   public void setCursorInfo(CursorInfo i) {
     if (i != cursorInfo) cursorInfo = i;
   }

  /**
   * <font face="Arial" size="2">
   * Adds the specified action listener. The action event is performed when the cursor
   * type has been changed for the target component.
   * @param <code>l</code> the specified action listener.
   * </font>
   */
   public void addActionListener (ActionListener l) {
     if (actionSupport != null) actionSupport = new ActionSupport();
     actionSupport.addListener(l);
   }

  /**
   * <font face="Arial" size="2">
   * Removes the specified action listener.
   * @param <code>l</code> the specified action listener.
   * </font>
   */
   public void removeActionListener (ActionListener l) {
     if (actionSupport != null) actionSupport.removeListener(l);
   }

  /**
   * <font face="Arial" size="2">
   * Gets the current cursor type.
   * @return a cursor type.
   * </font>
   */
   public int getCursorType() {
     return cursorType;
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
   * </font>
   */
   public void mouseDragged(MouseEvent e) {
     if (isPressed && !isDragged) isDragged = true;
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse button has been moved on a component
   * (with no buttons no down).
   * </font>
   */
   public void mouseMoved (MouseEvent e) {
     if (!isDragged) changeCursor(e.getX(), e.getY());
   }

  /**
   * <font face="Arial" size="2">
   * The method checks if the cursor type has to be changed for the specified location
   * relatively the target component and sets a new cursor type if it is necessary.
   * @param <code>x</code> the specified x cordinate relatively the target component.
   * @param <code>y</code> the specified y cordinate relatively the target component.
   * </font>
   */
   protected void changeCursor(int x, int y)
   {
     Component target = getTarget();
     int type = cursorInfo.getCursorType(target, x, y);
     if (type < 0)
     {
       if (defaultCursor != null)
       {
         target.setCursor(defaultCursor);
         setCursorType(defaultCursor.getType());
       }
     }
     else
     {
       Cursor cursor = target.getCursor();
       if (cursor == null) cursor = Cursor.getDefaultCursor();
       if (type != cursor.getType())
       {
         Cursor cc = new Cursor(type);
         target.setCursor(cc);
         setCursorType(cc.getType());
       }
     }
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse has been clicked on a component.
   * </font>
   */
   public void mouseClicked(MouseEvent e) {}

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse enters a component.
   * </font>
   */
   public void mouseEntered(MouseEvent e) {
     isExited = false;
     if (!isDragged) storeCursor();
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse exits a component.
   * </font>
   */
   public void mouseExited(MouseEvent e)  {
     isExited = true;
     if (!isDragged) restoreCursor();
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been pressed on a component.
   * </font>
   */
   public void mousePressed(MouseEvent e) {
     isPressed = true;
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been released on a component.
   * </font>
   */
   public void mouseReleased(MouseEvent e)
   {
     isPressed = false;
     if (isDragged)
     {
       isDragged = false;
       if (isExited) restoreCursor();
     }
   }

  /**
   * <font face="Arial" size="2">
   * The method is called whenever the target component has been changed. The method
   * implementation removes mouse and mouse motion listeners for the old target and
   * registers the listeners for the new target component.
   * @param <code>n</code> the specified new target component.
   * @param <code>o</code> the specified old target component.
   * </font>
   */
   protected void targetWasChanged(Component n, Component o)
   {
     if (o != null)
     {
       o.removeMouseListener(this);
       o.removeMouseMotionListener(this);
       restoreCursor();
     }

     if (n != null)
     {
       n.addMouseListener(this);
       n.addMouseMotionListener(this);
     }
   }

  /**
   * <font face="Arial" size="2">
   * Performs an action event.
   * </font>
   */
   protected void perform() {
     if (actionSupport != null) actionSupport.perform(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null));
   }

  /**
   * <font face="Arial" size="2">
   * Sets the specified cursor type. The method is used to store the new cursor type
   * in the internal field and perform an action event.
   * @param <code>type</code> the specified cursor type.
   * </font>
   */
   protected void setCursorType(int type)
   {
     if (type != cursorType)
     {
       cursorType = type;
       perform();
     }
   }

  /**
   * <font face="Arial" size="2">
   * Stores a current cursor type.
   * </font>
   */
   protected void storeCursor()
   {
     if (defaultCursor == null)
     {
       defaultCursor = getTarget().getCursor();
       if (defaultCursor == null) defaultCursor = Cursor.getDefaultCursor();
     }
   }

  /**
   * <font face="Arial" size="2">
   * Sets the cursor type. The method uses a cursor type that has been stored with
   * <code>storeCursor</code> method.
   * </font>
   */
   protected void restoreCursor()
   {
     if (defaultCursor != null)
     {
       getTarget().setCursor(defaultCursor);
       setCursorType(defaultCursor.getType());
       defaultCursor = null;
     }
   }
}
