/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp;

import java.awt.*;
import java.awt.event.*;
import org.zaval.awt.gdp.event.*;

/**
 * <font face="Arial" size="2">
 * This designer controller class that can be used to control size and location for the
 * specified target component. The controller works as follow:
 * <ul>
 *   <li>
 *     First of all, the controller is registered as a mouse listener for the target
 *     component.
 *   </li>
 *   <li>
 *     Creates designer container. The designer container is used to paint
 *     border around the target component and provide pos info.
 *   </li>
 *   <li>
 *     Registers shape controller for the designer container. The shape controller
 *     allows to control the container size and location using mouse.
 *   </li>
 * </ul>
 * <p>
 * If a mouse button has been pressed on the target component than the controller
 * tries to activate the controller. The process is described below:
 * <ul>
 *   <li>
 *     Checks if there is another designer controller that has been activated.
 *     If it is true, the activated controller will be de-activated.
 *   </li>
 *   <li>
 *     Initiates the designer container (DgnContainer) and adds the target component
 *     to the container.
 *   </li>
 *   <li>
 *     Adds the designer container to the parent of the target component.
 *   </li>
 * </ul>
 * Starting from the moment it is possible to resize and move the target component
 * using the mouse.
 * <p>
 * Use <code>addDgnControllerListener</code> and <code>removeDgnControllerListener</code>
 * methods to register or un-register DgnControllerListener. Using the interface it is
 * possible to listen when the designer controller has been activated or deactivated.
 * </font>
 */
public class DgnController
extends Controller
implements MouseListener
{
  /**
   * <font face="Arial" size="2">
   * This is activated designer controller.
   * </font>
   */
   private static DgnController current;

   private Container       design;
   private ShapeController shape;
   private boolean isPressed;
   private DgnControllerSupport support;

  /**
   * <font face="Arial" size="2">
   * Constructs a new designer controller with the specified target component.
   * @param <code>target</code> the specified target component.
   * </font>
   */
   public DgnController(Component target)
   {
     super(target);
     design = LibraryAdapter.getLibraryAdapter().makeDgnContainer();
     shape  = new ShapeController(design);
     shape.getCursorController().setCursorInfo(new DgnCursorInfo());
   }

  /**
   * <font face="Arial" size="2">
   * Adds the specified designer controller listener.
   * @param <code>l</code> the specified designer controller listener.
   * </font>
   */
   public void addDgnControllerListener(DgnControllerListener l) {
     if (support == null) support = new DgnControllerSupport();
     support.addListener (l);
   }

  /**
   * <font face="Arial" size="2">
   * Removes the specified designer controller listener.
   * @param <code>l</code> the specified designer controller listener.
   * </font>
   */
   public void removeDgnControllerListener(DgnControllerListener l) {
     if (support != null) support.removeListener (l);
   }

  /**
   * <font face="Arial" size="2">
   * Performs the designer event with the specified event id.
   * @param <code>id</code> the specified event id.
   * </font>
   */
   protected void perform (int id)  {
     if (support != null) support.perform (new DgnControllerEvent(getTarget(), id));
   }


  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been released on a component.
   * </font>
   */
   public void mouseReleased(MouseEvent e)
   {
     if (this != current && isPressed)
     {
       try {
         if (current != null) current.removeDgnContainer();
         addDgnContainer();
       }
       finally {
        isPressed = false;
       }
     }
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse has been clicked on a component.
   * </font>
   */
   public void mouseClicked (MouseEvent e) {}

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse enters a component.
   * </font>
   */
   public void mouseEntered (MouseEvent e) {}

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse exits a component.
   * </font>
   */
   public void mouseExited  (MouseEvent e) {}

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been pressed on a component.
   * </font>
   */
   public void mousePressed (MouseEvent e) {
     isPressed=true;
   }

  /**
   * <font face="Arial" size="2">
   * Moves the target to the new loaction.
   * @param <code>x</code> the new x coordinate of the target.
   * @param <code>y</code> the new y coordinate of the target.
   * </font>
   */
   public void moveTarget(int x, int y)
   {
     if (current == this)
     {
       Component target = getTarget();
       Point p1 = target.getLocation();
       design.setLocation(x - p1.x, y - p1.y);
     }
     else
       getTarget().setLocation (x, y);
   }

  /**
   * <font face="Arial" size="2">
   * Resizes the target to the specified width and height.
   * @param <code>w</code> the new x coordinate of the target.
   * @param <code>h</code> the new y coordinate of the target.
   * </font>
   */
   public void resizeTarget(int w, int h)
   {
     if (current == this)
     {
       Component target = getTarget();
       Dimension d1 = target.getSize();
       Dimension d2 = design.getSize();
       design.setSize(d2.width + w - d1.width, d2.height + h - d1.height);
     }
     getTarget().setSize (w, h);
   }

  /**
   * <font face="Arial" size="2">
   * Gets the designer container that is used with the controller.
   * @return designer container.
   * </font>
   */
   public Container getDgnContainer () {
     return design;
   }

  /**
   * <font face="Arial" size="2">
   * Deactivates current active designer controller, if it is exist.
   * </font>
   */
   public static synchronized void stopCurrentDgn() {
     if (current != null) current.removeDgnContainer();
   }

  /**
   * <font face="Arial" size="2">
   * Returns current active designer controller.
   * @return an active designer controller.
   * </font>
   */
   public static synchronized DgnController getCurrentDgn() {
     return current;
   }

  /**
   * <font face="Arial" size="2">
   * The method is called whenever the target component has been changed. The method
   * implementation removes mouse listener for the old target and registers the listener
   * for the new target component.
   * @param <code>n</code> the specified new target component.
   * @param <code>o</code> the specified old target component.
   * </font>
   */
   protected void targetWasChanged(Component n, Component o)
   {
     if (o != null)
     {
       o.removeMouseListener(this);
       if (current == this) removeDgnContainer();
     }
     if (n != null) n.addMouseListener(this);
   }

   private void addDgnContainer()
   {
     design.setEnabled (false);
     Component target = getTarget();
     Rectangle bounds = target.getBounds();
     Container parent = target.getParent();
     target.setVisible(false);
     design.setVisible(false);

     parent.remove(target);
     design.add   (target);
     parent.add   (design);

     Insets gaps = design.getInsets();
     design.setBounds(bounds.x - gaps.left, bounds.y - gaps.top,
                      bounds.width + gaps.left + gaps.right,
                      bounds.height + gaps.top + gaps.bottom);

     target.setVisible(true);
     design.setVisible(true);
     design.validate();

     design.setEnabled (true);
     target.requestFocus();
     current = this;

     perform (DgnControllerEvent.DGN_STARTED);
   }

   private void removeDgnContainer()
   {
     design.setEnabled (false);
     Component target = getTarget();
     Container parent = design.getParent();
     Rectangle bounds = design.getBounds();
     target.setVisible(false);
     design.setVisible(false);
     parent.remove(design);
     design.remove(target);
     Insets gaps = design.getInsets();
     parent.add(target);
     target.setLocation(bounds.x + gaps.left, bounds.y + gaps.top);
     target.setVisible(true);
     design.setEnabled (true);
     current = null;

     perform (DgnControllerEvent.DGN_FINISHED);
   }
}
