/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp;

import java.awt.*;
import java.awt.event.*;
import org.zaval.awt.gdp.event.*;

/**
 * <font face="Arial" size="2">
 * This controller is used to convert mouse events of the target component to
 * the mouse org.zaval.awt.gdp.event.MouseDragEvent events. To listen the events use
 * <code>addMouseDragListener</code> and <code>removeMouseDragListener</code> methods.
 * The main purpose of the class is to provide more handy method to handle mouse
 * drag events for the specified component.
 * </font>
 */
public class DragController
extends Controller
implements MouseMotionListener, MouseListener
{
   private Component        dragOwner;
   private MouseDragSupport dragSupport;
   private boolean isPressed;

  /**
   * <font face="Arial" size="2">
   * Constructs a new drag controller with the specified target component.
   * @param <code>target</code> the specified target component.
   * </font>
   */
   public DragController(Component target) {
     super(target);
   }

  /**
   * <font face="Arial" size="2">
   * Adds the specified drag listener.
   * @param <code>l</code> the specified drag listener.
   * </font>
   */
   public void addMouseDragListener (MouseDragListener l) {
     if (dragSupport == null) dragSupport = new MouseDragSupport();
     dragSupport.addListener(l);
   }

  /**
   * <font face="Arial" size="2">
   * Removes the specified drag listener.
   * @param <code>l</code> the specified drag listener.
   * </font>
   */
   public void removeMouseDragListener (MouseDragListener l) {
     if (dragSupport != null) dragSupport.removeListener(l);
   }

  /**
   * <font face="Arial" size="2">
   * Tests if the mouse is dragged at the moment.
   * @return <code>true</code> if the mouse is dragged; <code>false</code> otherwise.
   * </font>
   */
   public boolean isDragged () {
     return dragOwner != null;
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
   * </font>
   */
   public void mouseDragged(MouseEvent e)
   {
     if (isPressed)
     {
       if (!isDragged())
       {
         dragOwner = e.getComponent();
         perform(new MouseDragEvent(e.getComponent(), e.getX(), e.getY(), MouseDragEvent.DRAG_STARTED));
       }
       else
        perform(new MouseDragEvent(e.getComponent(), e.getX(), e.getY(), MouseDragEvent.DRAG_MOVED));
     }
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been released on a component.
   * </font>
   */
   public void mouseReleased(MouseEvent e)
   {
     try {
       if (isDragged() && isPressed)
       {
         dragOwner = null;
         perform(new MouseDragEvent(e.getComponent(), e.getX(), e.getY(), MouseDragEvent.DRAG_STOPPED));
       }
     }
     finally {
       isPressed = false;
     }
   }

  /**
   * <font face="Arial" size="2">
   * The method is called whenever the target component has been changed. The
   * implementation removes mouse and mouse motion listeners for the old target and
   * registers the listeners for the new target component.
   * @param <code>n</code> the specified new target component.
   * @param <code>o</code> the specified old target component.
   * </font>
   */
   protected void targetWasChanged(Component n, Component o)
   {
     if (o != null)
     {
       o.removeMouseListener(this);
       o.removeMouseMotionListener(this);
     }

     if (n != null)
     {
       n.addMouseListener(this);
       n.addMouseMotionListener(this);
     }
   }

  /**
   * <font face="Arial" size="2">
   * Performs the specified mouse drag event.
   * @param <code>e</code> the specified mouse drag event.
   * </font>
   */
   protected void perform(MouseDragEvent e) {
     if (dragSupport != null) dragSupport.perform (e);
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when a mouse button has been pressed on a component.
   * </font>
   */
   public void mousePressed(MouseEvent e) {
     isPressed = true;
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse button has been moved on a component
   * (with no buttons no down).
   * </font>
   */
   public void mouseMoved  (MouseEvent e) { }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse has been clicked on a component.
   * </font>
   */
   public void mouseClicked(MouseEvent e) { }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse enters a component.
   * </font>
   */
   public void mouseEntered(MouseEvent e) { }

  /**
   * <font face="Arial" size="2">
   * Invoked when the mouse exits a component.
   * </font>
   */
   public void mouseExited (MouseEvent e) { }
}

