/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp;

import java.awt.*;
import java.util.*;

/**
 * <font face="Arial" size="2">
 * This class is an implementation of CursorInfo interface. The class defines different cursors
 * types for nine areas of the specified component. The component area is devided as follow:
 * <br>
 * <table border="1">
 *   <tr>
 *      <td><font face="Arial" size="2">TLEFT</font></td>
 *      <td><font face="Arial" size="2">TOP</font></td>
 *      <td><font face="Arial" size="2">TRIGHT</font></td>
 *   </tr>
 *   <tr>
 *      <td><font face="Arial" size="2">LEFT</font></td>
 *      <td><font face="Arial" size="2">CENTER</font></td>
 *      <td><font face="Arial" size="2">RIGHT</font></td>
 *   </tr>
 *   <tr>
 *      <td><font face="Arial" size="2">BLEFT</font></td>
 *      <td><font face="Arial" size="2">BOTTOM</font></td>
 *      <td><font face="Arial" size="2">BRIGHT</font></td>
 *   </tr>
 * </table>
 * The size of the cells are defined by a top, left, right and bottom indents.
 * </font>
 */
public class InsetsCursorInfo
implements CursorInfo
{
 /**
  * <font face="Arial" size="2">
  * Defines left area type.
  * </font>
  */
  public static final int LEFT   = 1;

 /**
  * <font face="Arial" size="2">
  * Defines right area type.
  * </font>
  */
  public static final int RIGHT  = 2;

 /**
  * <font face="Arial" size="2">
  * Defines top area type.
  * </font>
  */
  public static final int TOP    = 4;

 /**
  * <font face="Arial" size="2">
  * Defines bottom area type.
  * </font>
  */
  public static final int BOTTOM = 8;

 /**
  * <font face="Arial" size="2">
  * Defines top-left area type.
  * </font>
  */
  public static final int TLEFT  = LEFT  | TOP;

 /**
  * <font face="Arial" size="2">
  * Defines top-right area type.
  * </font>
  */
  public static final int TRIGHT = RIGHT | TOP;

 /**
  * <font face="Arial" size="2">
  * Defines bottom-left area type.
  * </font>
  */
  public static final int BLEFT  = LEFT  | BOTTOM;

 /**
  * <font face="Arial" size="2">
  * Defines bottom-right area type.
  * </font>
  */
  public static final int BRIGHT = RIGHT | BOTTOM;

 /**
  * <font face="Arial" size="2">
  * Defines center area type.
  * </font>
  */
  public static final int CENTER = 0;

  private Insets insets = new Insets(6, 6, 6, 6);
  private Hashtable cursors = new Hashtable();

 /**
  * <font face="Arial" size="2">
  * Constructs a new insets cursor info.
  * </font>
  */
  public InsetsCursorInfo()
  {
    setCursorForArea (LEFT,   Cursor.W_RESIZE_CURSOR );
    setCursorForArea (RIGHT,  Cursor.E_RESIZE_CURSOR );
    setCursorForArea (TOP,    Cursor.N_RESIZE_CURSOR );
    setCursorForArea (BOTTOM, Cursor.S_RESIZE_CURSOR );
    setCursorForArea (TLEFT,  Cursor.NW_RESIZE_CURSOR);
    setCursorForArea (TRIGHT, Cursor.NE_RESIZE_CURSOR);
    setCursorForArea (BLEFT,  Cursor.SW_RESIZE_CURSOR);
    setCursorForArea (BRIGHT, Cursor.SE_RESIZE_CURSOR);
    setCursorForArea (CENTER, Cursor.MOVE_CURSOR);
  }

 /**
  * <font face="Arial" size="2">
  * Returns the cursor type for the specified component and location relatively the component.
  * The method using the specified location calculates what area contains the location and
  * returns a cursor type that is bound with the area.
  * @param <code>target</code> the specified component.
  * @param <code>x</code> the specified x coordinate relatively the component.
  * @param <code>y</code> the specified y coordinate relatively the component.
  * </font>
  */
  public int getCursorType (Component target, int x, int y)
  {
    int areaType = getAreaType(target, x, y);
    if (areaType >= 0) return getCursorForArea(areaType);
    return -1;
  }

 /**
  * <font face="Arial" size="2">
  * Binds the specified area with the specified cursor type. Use one of static constants of the class
  * as the area type. If the area type is undefined than the IllegalArgumentException will be thrown.
  * @param <code>t</code> the specified area type.
  * @param <code>c</code> the specified cursor type.
  * </font>
  */
  public void setCursorForArea(int t, int c) {
    if (!checkAreaType(t)) throw new IllegalArgumentException();
    cursors.put(new Integer(t), new Integer(c));
  }

 /**
  * <font face="Arial" size="2">
  * Returns the cursor type for the specified area type. If the area type is undefined than
  * the IllegalArgumentException will be thrown.
  * @return a cursor type.
  * </font>
  */
  public int getCursorForArea(int t)
  {
    if (!checkAreaType(t)) throw new IllegalArgumentException();
    Integer cursor = (Integer)cursors.get(new Integer(t));
    return cursor == null?-1:cursor.intValue();
  }

 /**
  * <font face="Arial" size="2">
  * Calculates and returns an area type for the specified component and the given location.
  * @param <code>c</code> the specified component.
  * @param <code>x</code> the specified x coordinate relatively the component.
  * @param <code>y</code> the specified y coordinate relatively the component.
  * @return an area type.
  * </font>
  */
  protected int getAreaType(Component c, int x, int y)
  {
    Dimension d = c.getSize();
    int       t = CENTER;
    if (x < insets.left) t |= LEFT;
    else
    if (x > (d.width - insets.right)) t |= RIGHT;

    if (y < insets.top) t |= TOP;
    else
    if (y > (d.height - insets.bottom)) t |= BOTTOM;

    return t;
  }

 /**
  * <font face="Arial" size="2">
  * Tests if the specified area type is valid. The method checks if the area type equals
  * one of the constants that are defined with the class.
  * @param <code>t</code> the specified area type.
  * @return <code>true</code> if the specified area type is valid; <code>false</code> otherwise.
  * </font>
  */
  protected boolean checkAreaType (int t) {
    return t == TOP || t == BOTTOM || t == LEFT || t == RIGHT ||
           t == TLEFT || t == TRIGHT || t == BRIGHT || t == BLEFT ||
           t == CENTER;
  }
}
