/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp;

import java.awt.*;

import org.zaval.awt.*;
import org.zaval.awt.gdp.event.*;

/**
 * <font face="Arial" size="2">
 * This shape controller class that can be used to control size and location for the specified
 * target component.
 * <p>
 * The controller registers itself as MouseDragListener listener of the target component and
 * registers a cursor controller for the target component.
 * </font>
 */
public class ShapeController
extends Controller
implements MouseDragListener
{
   private Container       topContainer;
   private Rectangle       abs   = new Rectangle();
   private Point           prev  = new Point();
   private Insets          state;
   private int             minWidth = 12, minHeight = 12;

   private CursorController cursor;
   private DragController   drag;

  /**
   * <font face="Arial" size="2">
   * Constructs a new shape controller with the specified target component.
   * @param <code>target</code> the specified target component.
   * </font>
   */
   public ShapeController(Component target)
   {
     super(target);
     drag = new DragController(target);
     drag.addMouseDragListener(this);
     setCursorController(new CursorController(target));
   }



  /**
   * <font face="Arial" size="2">
   * Invoked when first mouse drag events has been performed with the source component.
   * @param <code>e</code> the specified mouse drag event.
   * </font>
   */
   public void dragStarted(MouseDragEvent e)
   {
     state = getBoundsMask();
     if (state != null)
     {
       int x = e.getX(), y = e.getY();
       Component target = getTarget();
       Dimension d = target.getSize();
       topContainer = LibraryAdapter.getLibraryAdapter().getRootContainer(target);

       Point p = ComponentUtil.getAbsLocation(topContainer, target);
       abs.x      = p.x;
       abs.y      = p.y;
       abs.width  = d.width;
       abs.height = d.height;
       prev.x     = x;
       prev.y     = y;
       LibraryAdapter.getLibraryAdapter().drawRectOnComponent(topContainer, abs.x, abs.y, abs.width - 1, abs.height - 1, Color.black);
     }
   }

  /**
   * <font face="Arial" size="2">
   * Invoked every time when mouse drag event is performed with the source component.
   * @param <code>e</code> the specified mouse drag event.
   * </font>
   */
   public void dragMoved(MouseDragEvent e)
   {
     if (state != null)
     {
       int x = e.getX(), y = e.getY();
       Component target = getTarget();
       int dy = (y - prev.y);
       int dx = (x - prev.x);
       int absWidth  = abs.width  - dx * state.left + dx * state.right;
       int absHeight = abs.height - dy * state.top  + dy * state.bottom;
       if (absWidth >= minWidth && absHeight >= minHeight)
       {
         LibraryAdapter.getLibraryAdapter().drawRectOnComponent(topContainer, abs.x, abs.y, abs.width - 1, abs.height - 1, Color.black);
         abs.width  = absWidth;
         abs.height = absHeight;
         abs.y  = abs.y + dy * state.top;
         abs.x  = abs.x + dx * state.left;
         prev.x = x;
         prev.y = y;
         LibraryAdapter.getLibraryAdapter().drawRectOnComponent(topContainer, abs.x, abs.y, abs.width - 1, abs.height - 1, Color.black);
       }
     }
   }

  /**
   * <font face="Arial" size="2">
   * Invoked when mouse drag is stopped.
   * @param <code>e</code> the specified mouse drag event.
   * </font>
   */
   public void dragStopped(MouseDragEvent e)
   {
     if (state != null)
     {
       LibraryAdapter.getLibraryAdapter().drawRectOnComponent(topContainer, abs.x, abs.y, abs.width-1, abs.height-1, Color.black);
       Component target = (Component)e.getSource();
       if ((state.top + state.left + state.bottom + state.right) == 4)
         moveComponent(abs.x, abs.y);
       else
         resizeComponent(abs);
       topContainer = null;
     }
   }

  /**
   * <font face="Arial" size="2">
   * Gets the cursor controller that is used for the target component with the controller.
   * @return a cursor controller .
   * </font>
   */
   public CursorController getCursorController() {
     return cursor;
   }

  /**
   * <font face="Arial" size="2">
   * Sets the specified cursor controller.
   * @param <code>c</code> the specified cursor controller.
   * </font>
   */
   protected void setCursorController(CursorController c)
   {
     if (c != cursor)
     {
       if (cursor != null) cursor.setTarget(null);
       cursor = c;
       if (cursor != null) cursor.setTarget(getTarget());
     }
   }

  /**
   * <font face="Arial" size="2">
   * Moves the target component to the new location.
   * @param <code>ax</code> the specified x coordinate relatively the top-level parent
   * component of the target component.
   * @param <code>ay</code> the specified y coordinate relatively the top-level parent
   * component of the target component.
   * </font>
   */
   protected void moveComponent(int ax, int ay)
   {
     Component target = getTarget();
     Container parent = target.getParent();

     if (parent != null)
     {

        target.setVisible(false);
        parent.remove(target);
        Container c = ComponentUtil.getContainerAt(topContainer, ax, ay);

        if (c.getLayout() != null || isDgnComponent(c))
        {
          parent.add(target);
          target.setVisible(true);
          return;
        }

        Point p = new Point(ax, ay);
        if (c != topContainer) p = ComponentUtil.getRelLocation(ax, ay, topContainer, c);

        target.setLocation(p.x, p.y);
        c.add(target);
        c.invalidate();
        c.validate();
        target.setVisible(true);

        if (parent != c)
        {
          parent.invalidate();
          parent.validate();
        }
     }
   }

  /**
   * <font face="Arial" size="2">
   * Shapes the target component with the specified bounds.
   * @param <code>r</code> the new bounds for the target component. The <code>x</code> and
   * <code>y</code> fields of the bounds define a new location of the target relatively
   * the top-level parent component.
   * </font>
   */
   protected void resizeComponent(Rectangle r)
   {
     Component target = getTarget();

     Component parent = target.getParent();
     Point     p      = ComponentUtil.getRelLocation(r.x, r.y, topContainer, parent);
     target.setVisible(false);
     target.setLocation(p);
     target.setSize(r.width, r.height);
     target.setVisible(true);
     target.invalidate();
     target.validate();
   }

  /**
   * <font face="Arial" size="2">
   * Gets the insets mask. The insets indicates what bounds should be changed. It is necessary
   * to use 0 or 1 value as the insets fields values. The insets fileds meaning is described
   * bellow:
   * <ul>
   *   <li>
   *     <b>left</b> field indicates that x top-left corner coordinate of the target component
   *     should be changed according to the current mouse pointer location (a location of the
   *     bottom-right corner is not changed).
   *   </li>
   *   <li>
   *     <b>top</b> field indicates that y top-left corner coordinate of the target component
   *     should be changed according to the current mouse pointer location (a location of the
   *     bottom-right corner is not changed).
   *   </li>
   *   <li>
   *     <b>right</b> field indicates that x bottom-right corner coordinate of the target
   *     component should be changed according to the current mouse pointer location (a location
   *     of the top-left corner is not changed).
   *   </li>
   *   <li>
   *     <b>bottom</b> field indicates that y bottom-right corner coordinate of the target
   *     component should be changed according to the current mouse pointer location (a location
   *     of the top-left corner is not changed).
   *   </li>
   *   <li>
   *     If all fields are set to 1 than the component will be moved. In this case
   *     the target can get a new parent container.
   *   </li>
   *   <li>
   *     The insets is <code>null</code> if nothing should be changed for the target component.
   *   </li>
   * </ul>
   * <p>
   * The implementation defines the insets fields according to the current mouse cursor type
   * (the mouse cursor type is set with the cursor controller). The table bellow shows the
   * correspondence the cursor type and the insets that will be returnd.
   * <br>
   * <table border="1" width="100%">
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2"><b> Cursor type </b></font></td>
   *     <td align="Center"><font face="Arial" size="2"><b> Returned insets </b></font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.W_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=1, top=0, right=0, bottom=0</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.E_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=0, top=0, right=1, bottom=0</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.N_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=0, top=1, right=0, bottom=0</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.S_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=0, top=0, right=0, bottom=1</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.NW_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=1, top=1, right=0, bottom=0</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.NE_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=0, top=1, right=1, bottom=0</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.SW_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=1, top=0, right=0, bottom=1</font></td>
   *   </tr>
   *   <tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.SE_RESIZE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=0, top=0, right=1, bottom=1</font></td>
   *   </tr>
   *     <td align="Center"><font face="Arial" size="2">Cursor.MOVE_CURSOR</font></td>
   *     <td align="Center"><font face="Arial" size="2">left=1, top=1, right=1, bottom=1</font></td>
   *   </tr>
   * </table>
   * @return an insets.
   * </font>
   */
   protected Insets getBoundsMask()
   {
     if (cursor != null)
     {
       int type = cursor.getCursorType();
       Insets r = new Insets(0,0,0,0);
       switch (type)
       {
         case Cursor.W_RESIZE_CURSOR : r.left   = 1; break;
         case Cursor.E_RESIZE_CURSOR : r.right  = 1; break;
         case Cursor.N_RESIZE_CURSOR : r.top    = 1; break;
         case Cursor.S_RESIZE_CURSOR : r.bottom = 1; break;
         case Cursor.NW_RESIZE_CURSOR: r.top = 1; r.left = 1; break;
         case Cursor.NE_RESIZE_CURSOR: r.top = 1; r.right = 1; break;
         case Cursor.SW_RESIZE_CURSOR: r.bottom = 1; r.left = 1; break;
         case Cursor.SE_RESIZE_CURSOR: r.bottom = 1; r.right = 1; break;
         case Cursor.MOVE_CURSOR     : r = new Insets(1,1,1,1); break;
         default: r = null;
       }
       return r;
     }
     else
      return null;
   }

  /**
   * <font face="Arial" size="2">
   * The method is called whenever the target component has been changed. The controller
   * uses cursor controller and drag controller, so the method implementation synchronizes
   * the new target with the two controllers.
   * @param <code>n</code> the specified new target component.
   * @param <code>o</code> the specified old target component.
   * </font>
   */
   protected void targetWasChanged (Component n, Component o) {
     if (cursor != null) cursor.setTarget(n);
     if (drag   != null) drag.setTarget(n);
   }

   private static boolean isDgnComponent(Component c)
   {
     while (c != null) {
       if (c instanceof DgnComponent) return true;
       c = c.getParent();
     }
     return false;
   }
}


