/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.awt.gdp.event;

import java.awt.*;
import org.zaval.awt.*;
import org.zaval.awt.gdp.*;

/**
 * <font face="Arial" size="2">
 * This class represents mouse drag event.
 * </font>
 */
public class MouseDragEvent
extends org.zaval.util.event.EvObject
{
  /**
   * <font face="Arial" size="2">
   * The drag started event type.
   * </font>
   */
   public static final int DRAG_STARTED = 1;

  /**
   * <font face="Arial" size="2">
   * The drag stopped event type.
   * </font>
   */
   public static final int DRAG_STOPPED = 2;

  /**
   * <font face="Arial" size="2">
   * The drag moved event type.
   * </font>
   */
   public static final int DRAG_MOVED = 3;

   private int x, y;

  /**
   * <font face="Arial" size="2">
   * Constructs the event object with the specified source component, the given location
   * and the event id.
   * @param <code>src</code> the component where the event originated.
   * @param <code>x</code> the specified x coordinate relatively to the source component.
   * @param <code>y</code> the specified y coordinate relatively to the source component.
   * @param <code>id</code> the specified event id.
   * </font>
   */
   public MouseDragEvent(Component src, int x, int y, int id)
   {
     super(src, id);
     this.x = x;
     this.y = y;
   }

  /**
   * <font face="Arial" size="2">
   * Gets the x coordinate where the event originated relatively to the source component.
   * @return a x coordinate.
   * </font>
   */
   public int getX() {
     return x;
   }

  /**
   * <font face="Arial" size="2">
   * Gets the y coordinate where the event originated relatively to the source component.
   * @return an y coordinate.
   * </font>
   */
   public int getY() {
     return y;
   }

  /**
   * <font face="Arial" size="2">
   * Defines and returns destination component description. The class uses the mouse pointer
   * location to find the destination.
   * @return a destination object description.
   * </font>
   */
   public Destination getDestination()
   {
     Component src = (Component)getSource();
     Container top = LibraryAdapter.getLibraryAdapter().getRootContainer(src);
     Point     abs = ComponentUtil.getAbsLocation(top, src);

     if (abs != null)
     {
       abs.x += x;
       abs.y += y;
       Component dest = ComponentUtil.getComponentAt(top, abs.x, abs.y);
       Point p = ComponentUtil.getRelLocation(abs.x, abs.y, top,  dest);
       return new Destination (dest, p.x, p.y);
     }
     else
       return null;
   }

  /**
   * <font face="Arial" size="2">
   * The method returns <code>true</code> if the specified id is valid for the event,
   * otherwise returns <code>false</code>. The method is called with the event constructor
   * to test if the event id is valid and if the event is not valid the IllegalArgumentException
   * will be thrown.
   * @param <code>id</code> the specified id.
   * </font>
   */
   protected boolean checkID (int id) {
     return id == DRAG_STARTED || id == DRAG_STOPPED || id == DRAG_MOVED;
   }

  /**
   * <font face="Arial" size="2">
   * This class represents destination component. The destination component
   * is determined basing on the the mouse pointer location.
   * </font>
   */
   public class Destination
   {
      private Component destination;
      private int       x, y;

     /**
      * <font face="Arial" size="2">
      * Constructs the destination descriptor class witn the specified destination component
      * and the given location relatively the destination component.
      * @param <code>target</code> the specified destination component.
      * @param <code>x</code> the specified x coordinate relatively to the destination component.
      * @param <code>y</code> the specified y coordinate relatively to the destination component.
      * </font>
      */
      Destination(Component target, int x, int y)
      {
        destination = target;
        this.x = x;
        this.y = y;
      }

     /**
      * <font face="Arial" size="2">
      * Gets the destination component.
      * @return a destination component.
      * </font>
      */
      public Component getDestComponent() {
        return destination;
      }

     /**
      * <font face="Arial" size="2">
      * Gets the destination location relatively the destination component.
      * @return a destination location.
      * </font>
      */
      public Point getDestLocation() {
        return new Point(x, y);
      }
   }
}
