/**
 *     Caption: Zaval GUI Designer Package
 *     $Revision: 1.51 $
 *     $Date: 2002/06/06 14:30:17 $
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    1.0
 *
 * The Zaval GUI Designer Package (GDP) is set of components that allows to control
 * size, location, cursor types for the Java AWT/Swing components. Actually, the
 * components are Java AWT events listeners that have no GUI part.
 *
 * For more info on this product read Zaval GUI Designer Package Tutorial
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/gdp/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0004/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.util.event;

import java.util.*;

/**
 * <font face="Arial" size="2">
 * This abstract, utility class can be inherited with a classes that implement:
 * <ul>
 *   <li>
 *     Supporting list of listeners with a given type. Use <code>addListener</code>
 *     and <code>removeListener</code> methods to control the list content.
 *   </li>
 *   <li>
 *     Delivering events with a given type to the listeners. First of all, implement
 *     <code>perform(java.util.EventListener, java.util.EventObject)</code> method that
 *     determines how the specified event should be delivered to the listener and
 *     after that you can use <code>perform(java.util.EventObject)</code> to delivery
 *     the specified event to all listeners of the list.
 *   </li>
 * </ul>
 * <p>
 * The listener supporting class provides special mechanism to listen exceptions that
 * were thrown during events handling. For the purpose just implement AbortListner
 * interface with your listener interface.
 * </font>
 */
public abstract class ListenerSupport
{
   private Vector v;

  /**
   * <font face="Arial" size="2">
   * Adds the specified listener to the listeners list.
   * @param <code>l</code> the specified listener.
   * </font>
   */
   public void addListener(EventListener l)  {
     if (v == null) v = new Vector();
     if (!v.contains(l)) v.addElement(l);
   }

  /**
   * <font face="Arial" size="2">
   * Removes the specified listener from the listeners list.
   * @param <code>l</code> the specified listener.
   * </font>
   */
   public void removeListener(EventListener l)
   {
     if (v == null || !v.contains(l)) return;
     v.removeElement(l);
     if (v.size() == 0) v = null;
   }

  /**
   * <font face="Arial" size="2">
   * Fires the specified event to every member of the listeners list.
   * The method uses <code>perform(java.util.EventListener, java.util.EventObject)</code>
   * method to delivery the event to every listener.
   * @param <code>e</code> the specified event.
   * </font>
   */
   public void perform(EventObject e)
   {
     if (v == null) return;
     for (int i=0; i<v.size(); i++)
     {
       EventListener l = (EventListener)v.elementAt(i);
       try {
         perform(l, e);
       }
       catch (Exception ee) {
         if (l instanceof AbortListener) ((AbortListener)l).aborted(e, ee);
         else
           ee.printStackTrace();
       }
     }
   }

 /**
  * <font face="Arial" size="2">
  * The method knows how the specified event has to be delivered to an appropriate
  * method of the specified listener.
  * @param <code>l</code> the specified listener.
  * @param <code>e</code> the specified event.
  * </font>
  */
   public abstract void perform(EventListener l, EventObject e);
}
